#include "RegisterAllocator.h"
#include "BasicBlock.h"
#include "setOverloads.h"

void RegisterAllocator::analyseLive() {
	for (const auto& v : vars) {
		if (!v->info) continue;
		for (const auto& q : v->writes) {
			q->block->flow.addDef(v);
		}
		for (const auto& q : v->uses) {
			q->block->flow.addUse(v);
		}
	}

	for (const auto& b : blocks) {
		for (const auto& phiMap : b->phi->phi) {
			for (const auto& phiRow : phiMap) {
				b->flow.def.emplace(phiRow.first);
			}
		}
	}

	while (true) {
		bool changed = false;
		for (const auto& b : blocks) {
			set<VariablePtr> pIn, pOut, diff;
			auto &f = b->flow;
			pIn.swap(f.in);
			pOut.swap(f.out);
			f.in = f.use + (pOut - f.def);
			for (const auto& succ : b->out) {
				f.out += succ->flow.inForBlock(b);
			}
			if (f.in != pIn || f.out != pOut) changed = true;
		}
		if (!changed) break;
	}

	assert(blocks.empty() || blocks[0]->flow.in.empty());
}

void RegisterAllocator::allocate() {
	analyseLive();
}
